from typing import List, Optional, Set

import asyncio
import configparser
import math
import operator
from pathlib import Path

import aiohttp
from rich.console import Console
from rich.table import Table
from user_agent import generate_user_agent

from rdf_linkchecker.checkers import CONFIG_DEFAULTS

_SESSION_HEADER = {
    "Connection": "keep-alive",
    "Upgrade-Insecure-Requests": "1",
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
    "Sec-Fetch-Site": "same-origin",
    "Sec-Fetch-Mode": "navigate",
    "Sec-Fetch-User": "?1",
    "Sec-Fetch-Dest": "document",
    "Referer": "https://www.google.com/",
    "Accept-Encoding": "gzip, deflate, br",
    "Accept-Language": "en-US,en;q=0.9",
    "User-Agent": generate_user_agent(),
}


class Checker:
    """`requests` based link checker"""

    def __init__(self, configfile: Optional[Path] = None):
        self.config = configparser.ConfigParser()
        self.config.read_dict(CONFIG_DEFAULTS)
        if configfile:
            self.config.read(filenames=[configfile])
        self.urls: Set[str] = set()
        try:
            self.skip_domains = self.config["skip"]["domains"].split(",")
        except AttributeError:
            self.skip_domains = []
        self._session = None

    def _accept_url(self, url):
        for skip in self.skip_domains:
            if skip in url:
                return False
        return True

    def add_urls(self, urls: List[str]) -> None:
        upd = {u for u in urls if self._accept_url(u)}
        self.urls.update(upd)

    async def _check_single(self, url: str, session: aiohttp.ClientSession) -> bool:
        """Return True if resource request succeeded, else False

        Uses the streaming version to cut down on dl size/time
        """
        con = self.config["connection"]

        async def _check():
            try:
                timeout = aiohttp.ClientTimeout(total=int(con["timeout"]))
                async with session.get(url, timeout=timeout) as response:
                    response.raise_for_status()
                    return None
            except (aiohttp.ClientError, asyncio.TimeoutError) as e:
                return e

        sleep_ms = float(con["sleep"]) * 1000
        sleep_fac = 2
        retries = min(0, int(con["retries"]))
        for try_no in range(retries + 1):
            err = await _check()
            if err is None:
                return None
            await asyncio.sleep(
                math.ceil(sleep_ms * math.pow(sleep_fac, try_no) / 1000)
            )
        return err

    async def _check(self):
        if len(self.urls) == 0:
            # we cannot await an empty tasks list
            tasks = []
        else:
            async with aiohttp.ClientSession(headers=_SESSION_HEADER) as session:
                tasks = [
                    asyncio.ensure_future(self._check_single(u, session=session))
                    for u in self.urls
                ]
                await asyncio.wait(tasks)
        self.results = [t.result() for t in tasks]

    def check(self) -> bool:
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._check())
        if self.config["reporting"]["level"] != "none":
            self.report_results(self.results)
        return all(r is None for r in self.results)

    def report_results(self, results):
        rptg = self.config["reporting"]
        only_failed = rptg["level"] == "only-failed"

        title = "Failed URLs" if only_failed else "Checked URLs"
        table = Table("URL", "Ok?", "Reason", title=title)
        for url, error in sorted(zip(self.urls, results), key=operator.itemgetter(0)):
            if rptg["level"] == "all" or (only_failed and error is None):
                if error is None:
                    table.add_row(url, "[green]✓[/green]", "")
                else:
                    table.add_row(url, "[red]x[/red]", str(error))

        def _print(_console):
            if table.row_count:
                _console.print(table)

        if rptg["target"] != "console":
            with open(rptg["target"], "wt") as report_file:
                console = Console(file=report_file)
                _print(console)
        else:
            console = Console()
            _print(console)



# type: ignore[attr-defined]
from typing import List, Optional

import sys
from enum import Enum
from pathlib import Path

import typer
from rich import console

from rdf_linkchecker import version
from rdf_linkchecker.checkers.requests_based import Checker
from rdf_linkchecker.graph import get_urls


class Color(str, Enum):
    white = "white"
    red = "red"
    cyan = "cyan"
    magenta = "magenta"
    yellow = "yellow"
    green = "green"


app = typer.Typer(
    name="rdf-linkchecker",
    help="Awesome `rdf-linkchecker` is a Python cli/package created with https://github.com/TezRomacH/python-package-template",
    add_completion=False,
)


def version_callback(print_version: bool) -> None:
    """Print the version of the package."""
    if print_version:
        console.Console().print(
            f"[yellow]rdf-linkchecker[/] version: [bold blue]{version}[/]"
        )
        raise typer.Exit()


@app.command(name="")
def main(
    config_file: Optional[Path] = typer.Option(
        None,
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        resolve_path=True,
    ),
    files: List[Path] = typer.Argument(
        ...,
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        resolve_path=True,
    ),
    print_version: bool = typer.Option(
        None,
        "-v",
        "--version",
        callback=version_callback,
        is_eager=True,
        help="Prints the version of the rdf-linkchecker package.",
    ),
) -> None:
    """Check URLs in given RDF files"""
    checker = Checker(config_file)
    checker.add_urls(get_urls(files))
    sys.exit(int(checker.check()) - 1)


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rdf_linkchecker', 'rdf_linkchecker.checkers']

package_data = \
{'': ['*']}

install_requires = \
['Brotli>=1.0.9,<2.0.0',
 'aiohttp>=3.8.1,<4.0.0',
 'rdflib>=6.1.1,<7.0.0',
 'rich>=11.0.0,<12.0.0',
 'typer[all]>=0.4.0,<0.5.0',
 'user-agent>=0.1.10,<0.2.0',
 'validators>=0.18.2,<0.19.0']

extras_require = \
{':python_version < "3.8"': ['importlib_metadata>=4.5.0,<5.0.0']}

entry_points = \
{'console_scripts': ['rdf-linkchecker = rdf_linkchecker.__main__:app']}

setup_kwargs = {
    'name': 'rdf-linkchecker',
    'version': '3.0.0',
    'description': 'Awesome `rdf-linkchecker` is a Python cli/package created with https://github.com/TezRomacH/python-package-template',
    'long_description': '# rdf_linkchecker\n\n<div align="center">\n\n[![Build status](https://github.com/WWU-AMM/rdf_linkchecker/workflows/build/badge.svg?branch=main&event=push)](https://github.com/WWU-AMM/rdf_linkchecker/actions?query=workflow%3Abuild)\n[![Python Version](https://img.shields.io/pypi/pyversions/rdf_linkchecker.svg)](https://pypi.org/project/rdf_linkchecker/)\n[![Dependencies Status](https://img.shields.io/badge/dependencies-up%20to%20date-brightgreen.svg)](https://github.com/WWU-AMM/rdf_linkchecker/pulls?utf8=%E2%9C%93&q=is%3Apr%20author%3Aapp%2Fdependabot)\n\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Security: bandit](https://img.shields.io/badge/security-bandit-green.svg)](https://github.com/PyCQA/bandit)\n[![Pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/WWU-AMM/rdf_linkchecker/blob/main/.pre-commit-config.yaml)\n[![Semantic Versions](https://img.shields.io/badge/%20%20%F0%9F%93%A6%F0%9F%9A%80-semantic--versions-e10079.svg)](https://github.com/WWU-AMM/rdf_linkchecker/releases)\n[![License](https://img.shields.io/github/license/WWU-AMM/rdf_linkchecker)](https://github.com/WWU-AMM/rdf_linkchecker/blob/main/LICENSE)\n![Coverage Report](assets/images/coverage.svg)\n\nCheck URLs in Triple-Store files.\n\n</div>\n\n## Installation\n\n```bash\npython3 -m pip install rdf-linkchecker\n```\n\n## Usage\n\nRun with a list of files to check\n\n```bash\nrdf-linkchecker myonefile.ttl myotherfile.ttl\n```\n\n## Configuration\n\nThe `--config-filename` option can be used to pass an `INI`-style file.\n```\n[connection]\nretries = 1\n#seconds\ntimeout = 3\n# first sleep in seconds between retries\n# exponential backoff afterwards\nsleep =1\n\n[skip]\n# any URL matching an entry in this comma separated list is not checked\ndomains = https://some.tld/path,http://someother.tld\n\n# level: "all" | "only-failed" | "none"\n# target: "console" | filename\n"reporting": {"level": "all", "target": "console"}\n```\n\n\n\n## Credits [![🚀 Your next Python package needs a bleeding-edge project structure.](https://img.shields.io/badge/python--package--template-%F0%9F%9A%80-brightgreen)](https://github.com/TezRomacH/python-package-template)\n\nThis project was generated with [`python-package-template`](https://github.com/TezRomacH/python-package-template)\n',
    'author': 'WWU-AMM',
    'author_email': 'rene.fritze@wwu.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/renefritze/rdf-linkchecker',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
